using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace JelliesAPlenty
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        enum GameState
        {
            Running,
            NextLevel,
            SharkReleased,
            Eaten,
            GameOver,
        }
        private float mTimeInState = 0;
        GameState mCurrentState;

        enum JellyState
        {
            Sitting,
            PreJump,
            UpArc,
            DownArc,
            Landing
        }
        JellyState mCurrentJellyState = JellyState.Sitting;

        //The X, Y positions of the Jelly prior to moving to the next square
        int mJellyStateStartX = 215;
        int mJellyStateStartY = 190;

        Sprite.SpriteManagement mSprites;
        Text.TextManagement mDisplayText;
        JelliesAPlenty.GameData mGameData;

        Random mRandomGenerate = new Random();

        //Keyboard
        bool m_SpacePressed = false;
        bool m_SpacePressedBefore = false;

        float mSharkMouthElapsed = (float)0.3;
        float mMoveSharkElapsed = (float)0.5;
        float mMoveJellyElapsed = (float)0.05;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);

            graphics.PreferredBackBufferHeight = 600;
            graphics.PreferredBackBufferWidth = 800;

            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            LoadResources();
            StartGame();
        }

        //Load all of the game's resources (sprites, sounds, etc.)
        protected void LoadResources()
        {
            LoadGameData();
            LoadTextures();
            LoadText();
        }//End LoadResources()

        //Load all of the textures and sprites to be used in the game
        protected void LoadTextures()
        {
            //Initialize the Sprite Management class
            mSprites = new Sprite.SpriteManagement();

            // TODO: Load any ResourceManagementMode.Automatic content

            //Add the textures to the Sprite Management class
            mSprites.Textures.Add("Jelly", Content.Load<Texture2D>("Images/JumpingJelly") as Texture2D);
            mSprites.Textures.Add("Background", Content.Load<Texture2D>("Images/Background") as Texture2D);
            mSprites.Textures.Add("Highlight", Content.Load<Texture2D>("Images/Highlight") as Texture2D);
            mSprites.Textures.Add("Computer", Content.Load<Texture2D>("Images/Computer") as Texture2D);
            mSprites.Textures.Add("ComputerPanel", Content.Load<Texture2D>("Images/ComputerPanel") as Texture2D);
            mSprites.Textures.Add("ColorStrip", Content.Load<Texture2D>("Images/ColorStrip") as Texture2D);
            mSprites.Textures.Add("Warning", Content.Load<Texture2D>("Images/Warning") as Texture2D);
            mSprites.Textures.Add("BackgroundGrey", Content.Load<Texture2D>("Images/BackgroundGrey") as Texture2D);
            mSprites.Textures.Add("Cage", Content.Load<Texture2D>("Images/Cage") as Texture2D);
            mSprites.Textures.Add("CageDoor", Content.Load<Texture2D>("Images/CageDoor") as Texture2D);
            mSprites.Textures.Add("RedEyes", Content.Load<Texture2D>("Images/RedEyes") as Texture2D);
            mSprites.Textures.Add("SharkGuy1", Content.Load<Texture2D>("Images/SharkGuy1") as Texture2D);
            mSprites.Textures.Add("SharkGuy2", Content.Load<Texture2D>("Images/SharkGuy2") as Texture2D);
            mSprites.Textures.Add("Jar", Content.Load<Texture2D>("Images/Jar") as Texture2D);
            mSprites.Textures.Add("JellyBeans", Content.Load<Texture2D>("Images/JellyBeans") as Texture2D);
            mSprites.Textures.Add("WhiteBox", Content.Load<Texture2D>("Images/White") as Texture2D);

            //Add the Sprites
            mSprites.Sprites.Add("Computer", new Sprite.Sprite("Computer", new Rectangle(198, 0, 250, 250), true));
            mSprites.Sprites.Add("ColorSquare1", new Sprite.Sprite("ColorStrip", new Rectangle(230, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare2", new Sprite.Sprite("ColorStrip", new Rectangle(245, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare3", new Sprite.Sprite("ColorStrip", new Rectangle(260, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare4", new Sprite.Sprite("ColorStrip", new Rectangle(275, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare5", new Sprite.Sprite("ColorStrip", new Rectangle(290, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare6", new Sprite.Sprite("ColorStrip", new Rectangle(230, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare7", new Sprite.Sprite("ColorStrip", new Rectangle(245, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare8", new Sprite.Sprite("ColorStrip", new Rectangle(260, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare9", new Sprite.Sprite("ColorStrip", new Rectangle(275, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare10", new Sprite.Sprite("ColorStrip", new Rectangle(290, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare11", new Sprite.Sprite("ColorStrip", new Rectangle(305, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare12", new Sprite.Sprite("ColorStrip", new Rectangle(320, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare13", new Sprite.Sprite("ColorStrip", new Rectangle(337, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare14", new Sprite.Sprite("ColorStrip", new Rectangle(352, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare15", new Sprite.Sprite("ColorStrip", new Rectangle(367, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare16", new Sprite.Sprite("ColorStrip", new Rectangle(305, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare17", new Sprite.Sprite("ColorStrip", new Rectangle(320, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare18", new Sprite.Sprite("ColorStrip", new Rectangle(337, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare19", new Sprite.Sprite("ColorStrip", new Rectangle(352, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare20", new Sprite.Sprite("ColorStrip", new Rectangle(367, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare21", new Sprite.Sprite("ColorStrip", new Rectangle(382, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare22", new Sprite.Sprite("ColorStrip", new Rectangle(399, 138, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare23", new Sprite.Sprite("ColorStrip", new Rectangle(382, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ColorSquare24", new Sprite.Sprite("ColorStrip", new Rectangle(399, 150, 15, 15), new Rectangle(GenerateColorSquare(), 0, 31, 31), true));
            mSprites.Sprites.Add("ComputerPanel", new Sprite.Sprite("ComputerPanel", new Rectangle(215, 135, 210, 33), true));
            mSprites.Sprites.Add("BackgroundGrey", new Sprite.Sprite("BackgroundGrey", new Rectangle(75, 0, mSprites.Textures["BackgroundGrey"].Width, mSprites.Textures["BackgroundGrey"].Height), true));
            mSprites.Sprites.Add("Highlight", new Sprite.Sprite("Highlight", new Rectangle(0, 0, 75, 75)));
            mSprites.Sprites.Add("Background", new Sprite.Sprite("Background", new Rectangle(75, 0, mSprites.Textures["Background"].Width, mSprites.Textures["Background"].Height), true));
            mSprites.Sprites.Add("Jelly", new Sprite.Sprite("Jelly", new Rectangle(190, 215, 50, 50), new Rectangle(0, 0, 256, mSprites.Textures["Jelly"].Height), true, new Vector2(1, 1)));
            mSprites.Sprites.Add("Life1", new Sprite.Sprite("Jelly", new Rectangle(350, 440, 30, 30), new Rectangle(0, 0, 256, mSprites.Textures["Jelly"].Height), true));
            mSprites.Sprites.Add("Life2", new Sprite.Sprite("Jelly", new Rectangle(380, 440, 30, 30), new Rectangle(0, 0, 256, mSprites.Textures["Jelly"].Height), true));
            mSprites.Sprites.Add("Life3", new Sprite.Sprite("Jelly", new Rectangle(410, 440, 30, 30), new Rectangle(0, 0, 256, mSprites.Textures["Jelly"].Height), true));
            mSprites.Sprites.Add("Cage", new Sprite.Sprite("Cage", new Rectangle(400, 170, 200, 245), true));
            mSprites.Sprites.Add("RedEye1", new Sprite.Sprite("RedEyes", new Rectangle(470, 210, 10, 20)));
            mSprites.Sprites.Add("RedEye2", new Sprite.Sprite("RedEyes", new Rectangle(470, 285, 10, 20)));
            mSprites.Sprites.Add("RedEye3", new Sprite.Sprite("RedEyes", new Rectangle(470, 357, 10, 20)));
            mSprites.Sprites.Add("CageDoor1", new Sprite.Sprite("CageDoor", new Rectangle(430, 189, 85, 65), true));
            mSprites.Sprites.Add("CageDoor2", new Sprite.Sprite("CageDoor", new Rectangle(430, 264, 85, 65), true));
            mSprites.Sprites.Add("CageDoor3", new Sprite.Sprite("CageDoor", new Rectangle(430, 334, 85, 65), true));
            mSprites.Sprites.Add("SharkGuy2", new Sprite.Sprite("SharkGuy2", new Rectangle(0, 0, 100, 100), new Vector2(1, 1)));
            mSprites.Sprites.Add("SharkGuy1", new Sprite.Sprite("SharkGuy1", new Rectangle(0, 0, 100, 100), new Vector2(1, 1)));
            mSprites.Sprites.Add("Warning", new Sprite.Sprite("Warning", new Rectangle(0, 0, this.Window.ClientBounds.Width, this.Window.ClientBounds.Height)));

            //Add Jelly Bean sprites
            for (int aIndex = 0; aIndex < mGameData.CustomData["JellyBeansMax"]; aIndex++)
            {
                string aKey = "JellyBean" + (aIndex + 1).ToString();
                mSprites.Sprites.Add(aKey, new Sprite.Sprite("JellyBeans", new Rectangle(0, 0, 75, 50), new Rectangle(0, 0, mSprites.Textures["JellyBeans"].Width, 77)));
            }//Next (int aIndex = 0; aIndex < mJellyBeansMax; aIndex++)

            mSprites.Sprites.Add("Jar", new Sprite.Sprite("Jar", new Rectangle(-5, 175, 210, 300), true));
            mSprites.Sprites.Add("GameOver", new Sprite.Sprite("WhiteBox", new Rectangle(this.Window.ClientBounds.Width / 2 - mSprites.Textures["WhiteBox"].Width / 2 - 100, this.Window.ClientBounds.Height / 2 - mSprites.Textures["WhiteBox"].Height / 2, mSprites.Textures["WhiteBox"].Width + 200, mSprites.Textures["WhiteBox"].Height)));
            mSprites.Sprites.Add("NextLevel", new Sprite.Sprite("WhiteBox", new Rectangle(this.Window.ClientBounds.Width / 2 - mSprites.Textures["WhiteBox"].Width / 2 - 100, this.Window.ClientBounds.Height / 2 - mSprites.Textures["WhiteBox"].Height / 2, mSprites.Textures["WhiteBox"].Width + 200, mSprites.Textures["WhiteBox"].Height)));

            //Set the Visible times for Sprites that are only visible for certain stretches of time
            mSprites.Sprites["Warning"].VisibleTime = 1;

            mSprites.Sprites["RedEye1"].VisibleTime = 5;
            mSprites.Sprites["RedEye1"].Flash = true;
            mSprites.Sprites["RedEye1"].Visible = true;

            mSprites.Sprites["RedEye2"].VisibleTime = 7;
            mSprites.Sprites["RedEye2"].Flash = true;
            mSprites.Sprites["RedEye2"].Visible = true;

            mSprites.Sprites["RedEye3"].VisibleTime = 4;
            mSprites.Sprites["RedEye3"].Flash = true;
            mSprites.Sprites["RedEye3"].Visible = true;

            for (int aIndex = 1; aIndex < 25; aIndex++)
            {
                string aKey = "ColorSquare" + aIndex.ToString();
                mSprites.Sprites[aKey].VisibleTime = (float)mRandomGenerate.Next(2, 9) / 10;
                mSprites.Sprites[aKey].Visible = true;
            }//Next (int aIndex = 1; aIndex < 25; aIndex++)

            mSprites.Sprites["Jelly"].Moving = false;
            mSprites.Sprites["Jelly"].Direction = JelliesAPlenty.Sprite.Sprite.eDirection.None;

        }//End LoadTextures()

        //Load all of the text objects to be displayed
        protected void LoadText()
        {
            //Initialize the Text Management class
            mDisplayText = new Text.TextManagement(this.graphics.GraphicsDevice, Content);

            //Initialize Text objects
            mDisplayText.Texts.Add("Lives", new Text.Text(new Vector2(240, 450), "LIVES*", 15));
            mDisplayText.Texts.Add("Level", new Text.Text(new Vector2(420, 20), "LEVEL:", 15));
            mDisplayText.Texts.Add("LevelValue", new Text.Text(new Vector2(530, 20), mGameData.Level.ToString(), 15));
            mDisplayText.Texts.Add("Score", new Text.Text(new Vector2(420, 40), "SCORE:", 15));
            mDisplayText.Texts.Add("ScoreValue", new Text.Text(new Vector2(530, 40), mGameData.Score.ToString(), 15));
            mDisplayText.Texts.Add("JellyBeans", new Text.Text(new Vector2(5, 20), "JELLY BEANS", 15));
            mDisplayText.Texts.Add("JellyBeansValue", new Text.Text(new Vector2(30, 40), mGameData.CustomData["JellyBeans"].ToString() + " OF " + mGameData.CustomData["JellyBeansMax"].ToString(), 15));
            mDisplayText.Texts.Add("QuestionValue1", new Text.Text(new Vector2(265, 60), " ", 35));
            mDisplayText.Texts.Add("QuestionSymbol", new Text.Text(new Vector2(305, 60), "+", 35));
            mDisplayText.Texts.Add("QuestionValue2", new Text.Text(new Vector2(345, 60), " ", 35));
            mDisplayText.Texts.Add("NextLevel", new Text.Text(new Vector2((this.Window.ClientBounds.Width / 2) - 200, (this.Window.ClientBounds.Height / 2) - 15), "LEVEL " + mGameData.Level.ToString(), 50));
            mDisplayText.Texts.Add("GameOver", new Text.Text(new Vector2(this.Window.ClientBounds.Width / 2 - 235, this.Window.ClientBounds.Height / 2 - 25), "GAME OVER", 50));
            mDisplayText.Texts.Add("PressSpace", new Text.Text(new Vector2(this.Window.ClientBounds.Width / 2 - 235, this.Window.ClientBounds.Height / 2 + 30), "PRESS SPACE TO START OVER OR ESCAPE TO EXIT", 8));
            mDisplayText.Texts.Add("Square1", new Text.Text(new Vector2(225, 200), " ", 35));
            mDisplayText.Texts.Add("Square2", new Text.Text(new Vector2(300, 200), " ", 35));
            mDisplayText.Texts.Add("Square3", new Text.Text(new Vector2(375, 200), " ", 35));
            mDisplayText.Texts.Add("Square4", new Text.Text(new Vector2(225, 275), " ", 35));
            mDisplayText.Texts.Add("Square5", new Text.Text(new Vector2(300, 275), " ", 35));
            mDisplayText.Texts.Add("Square6", new Text.Text(new Vector2(375, 275), " ", 35));
            mDisplayText.Texts.Add("Square7", new Text.Text(new Vector2(225, 350), " ", 35));
            mDisplayText.Texts.Add("Square8", new Text.Text(new Vector2(300, 350), " ", 35));
            mDisplayText.Texts.Add("Square9", new Text.Text(new Vector2(375, 350), " ", 35));

            //This text doesn't get displayed in the batch draw, it gets displayed at a different point 
            //so make it invisible for now.
            mDisplayText.Texts["NextLevel"].Visible = false;
            mDisplayText.Texts["GameOver"].Visible = false;
            mDisplayText.Texts["PressSpace"].Visible = false;

        }//End LoadText()

        //Load the game data associated with the game
        protected void LoadGameData()
        {
            //Initialize the game data object
            mGameData = new JelliesAPlenty.GameData();

            //Add some custom game data
            mGameData.CustomData.Add("JellyBeans", 0);
            mGameData.CustomData.Add("JellyBeansMax", 10);
            mGameData.CustomData.Add("QuestionPoints", 100);
        }//End LoadGameData()

        //Initialize all of the game data parameters to start (or re-start) the game
        protected void StartGame()
        {
            mSprites.Sprites["Jelly"].GridPosition = new Vector2(1, 1);

            mGameData.Lives = 3;
            mGameData.Score = 0;
            mGameData.Level = 1;
            mGameData.CustomData["JellyBeans"] = 0;

            mDisplayText.Texts["NextLevel"].Value = "LEVEL " + mGameData.Level.ToString();
            mDisplayText.Texts["NextLevel"].VisibleTime = 1;
            mDisplayText.Texts["NextLevel"].Visible = true;

            mSprites.Sprites["NextLevel"].VisibleTime = 1;
            mSprites.Sprites["NextLevel"].Visible = true;

            mSprites.Sprites["GameOver"].Visible = false;

            //Hide all of the Jelly Beans
            for (int index = 0; index < mGameData.CustomData["JellyBeansMax"]; index++)
            {
                string aKey = "JellyBean" + (index + 1).ToString();
                mSprites.Sprites[aKey].Visible = false;
            }

            GenerateNextQuestion();

            CurrentState = GameState.Running;
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // The time since Update was called last
            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;

            UpdateText();
            UpdateSprite(elapsed);
            UpdateSharkCreature(elapsed);
            UpdateComputerPanel(elapsed);

            UpdateState(elapsed);

            mSprites.Update(elapsed);
            mDisplayText.Update(elapsed);

            UpdateWarning();
            UpdateJellyBeans();
            UpdateKeyboard();

            //If the player has been eaten, then the question won't be displayed
            //for a particular elapsed time. Once that time has elapsed, the 
            //question text will be displayed again.
            if (CurrentState == GameState.Eaten)
            {
                mDisplayText.Texts["QuestionValue1"].Visible = false;
                mDisplayText.Texts["QuestionValue2"].Visible = false;
                mDisplayText.Texts["QuestionSymbol"].Visible = false;
            }
            else
            {
                mDisplayText.Texts["QuestionValue1"].Visible = true;
                mDisplayText.Texts["QuestionValue2"].Visible = true;
                mDisplayText.Texts["QuestionSymbol"].Visible = true;
            }

            //If the Jelly is jumping, don't display the Highlight. Only display
            //the Highlight when the Jelly is resting on a Square.
            if (mSprites.Sprites["Jelly"].Moving == false)
            {
                //Make Highlight visible
                mSprites.Sprites["Highlight"].Position = new Vector2((206 - 75) + (mSprites.Sprites["Jelly"].GridPositionX * 75), (180 - 75) + (mSprites.Sprites["Jelly"].GridPositionY * 75));
                mSprites.Sprites["Highlight"].Visible = true;
            }
            else
            {
                mSprites.Sprites["Highlight"].Visible = false;
            }

            //Update the location of the Jelly
            if (mSprites.Sprites["Jelly"].Moving == false)
            {
                mSprites.Sprites["Jelly"].Position = new Vector2((215 - 75) + (mSprites.Sprites["Jelly"].GridPositionX * 75), (190 - 75) + (mSprites.Sprites["Jelly"].GridPositionY * 75));
            }

            //Draw the current number of lives
            mSprites.Sprites["Life1"].Visible = false;
            mSprites.Sprites["Life2"].Visible = false;
            mSprites.Sprites["Life3"].Visible = false;
            for (int aIndex = 0; aIndex < mGameData.Lives; aIndex++)
            {
                string aKey = "Life" + (aIndex + 1).ToString();
                mSprites.Sprites[aKey].Visible = true;
            }

            //Red Eyes
            if (mSprites.Sprites["Warning"].Visible == true)
            {
                //Display all 3 red eyes
                mSprites.Sprites["RedEye1"].Visible = true;
                mSprites.Sprites["RedEye2"].Visible = true;
                mSprites.Sprites["RedEye3"].Visible = true;
            }

            //Display the Shark with his mouth opened or closed           
            if (CurrentState == GameState.SharkReleased)
            {
                mSprites.Sprites["SharkGuy1"].Position = new Vector2((190 - 75) + (mSprites.Sprites["SharkGuy1"].GridPositionX * 75), (170 - 75) + (mSprites.Sprites["SharkGuy1"].GridPositionY * 75));
                mSprites.Sprites["SharkGuy2"].Position = new Vector2((190 - 75) + (mSprites.Sprites["SharkGuy1"].GridPositionX * 75), (170 - 75) + (mSprites.Sprites["SharkGuy1"].GridPositionY * 75));
            }
            else
            {
                mSprites.Sprites["SharkGuy1"].Visible = false;
                mSprites.Sprites["SharkGuy2"].Visible = false;
            }

            if (CurrentState == GameState.SharkReleased)
            {
                if (mSprites.Sprites["SharkGuy1"].GridPositionX == mSprites.Sprites["Jelly"].GridPositionX && mSprites.Sprites["SharkGuy1"].GridPositionY == mSprites.Sprites["Jelly"].GridPositionY)
                {
                    CurrentState = GameState.Eaten;
                }
            }

            base.Update(gameTime);
        }

        private void UpdateState(float elapsed)
        {
            mTimeInState += elapsed;
            switch (CurrentState)
            {
                case GameState.Running:
                    {
                        break;
                    }

                case GameState.Eaten:
                    {
                        if (mTimeInState > (float)1.0)
                        {
                            CurrentState = GameState.Running;
                        }
                        break;
                    }

                case GameState.SharkReleased:
                    {
                        break;
                    }

                case GameState.NextLevel:
                    {
                        break;
                    }


                case GameState.GameOver:
                    {
                        break;
                    }
            }

        }

        private GameState CurrentState
        {
            get
            {
                return mCurrentState;
            }
            set
            {
                mCurrentState = value;
                mTimeInState = 0;
                switch (mCurrentState)
                {
                    case GameState.Eaten:
                        {
                            Eaten();
                            break;
                        }

                    case GameState.GameOver:
                        {
                            //GameOver();
                            break;
                        }

                    case GameState.NextLevel:
                        {
                            //NextLevel();
                            break;
                        }

                    case GameState.Running:
                        {
                            //Running();
                            break;
                        }

                    case GameState.SharkReleased:
                        {
                            //SharkRelease();
                            mSprites.Sprites["SharkGuy2"].Visible = true;
                            break;
                        }
                }
            }
        }

        private void Eaten()
        {
            mSprites.Sprites["Jelly"].GridPosition = new Vector2(1, 1);
            mGameData.Lives -= 1;
            if (mGameData.Lives < 0)
            {
                mGameData.Lives = 0;
                CurrentState = GameState.GameOver;
            }
        }


        //Flash the warning if appropriate
        private void UpdateWarning()
        {
            //If the warning is still being displayed (elapsed > 0), then flash it 
            //by setting it's Visible property True/False
            if (mSprites.Sprites["Warning"].Elapsed > 0)
            {
                mSprites.Sprites["Warning"].Visible = !mSprites.Sprites["Warning"].Visible;
            }
            else
            {
                mSprites.Sprites["Warning"].Visible = false;
            }//End if (mSprites.Sprites["Warning"].Elapsed > 0)
        }//End UpdateWarning

        private void UpdateJellyBeans()
        {
            for (int aIndex = 0; aIndex < mGameData.CustomData["JellyBeansMax"]; aIndex++)
            {
                string aKey = "JellyBean" + (aIndex + 1).ToString();

                if (mSprites.Sprites[aKey].Visible == true && mSprites.Sprites[aKey].Moving == true)
                {
                    mSprites.Sprites[aKey].Position = new Vector2(mSprites.Sprites[aKey].Position.X, mSprites.Sprites[aKey].Position.Y + 1);
                    if (mSprites.Sprites[aKey].Position.Y > 400)
                    {
                        mSprites.Sprites[aKey].Moving = false;
                    }
                    else
                    {
                        for (int aCheck = 0; aCheck < aIndex; aCheck++)
                        {
                            string aComparisonKey = "JellyBean" + (aCheck + 1).ToString();
                            if (mSprites.Sprites[aKey].Position.Y > mSprites.Sprites[aComparisonKey].Position.Y - 50 && mSprites.Sprites[aKey].Position.X == mSprites.Sprites[aComparisonKey].Position.X)
                            {
                                mSprites.Sprites[aKey].Moving = false;
                            }
                        }

                    }

                }
            }
        }

        private void UpdateComputerPanel(float elapsed)
        {
            for (int aIndex = 1; aIndex < 25; aIndex++)
            {
                string aKey = "ColorSquare" + aIndex.ToString();

                if (mSprites.Sprites[aKey].Visible == false)
                {
                    mSprites.Sprites[aKey].TexturePositionX = GenerateColorSquare();
                    mSprites.Sprites[aKey].Visible = true;
                }
            }
        }

        private void UpdateSharkCreature(float elapsed)
        {
            if (CurrentState == GameState.SharkReleased)
            {
                mMoveSharkElapsed -= elapsed;
                if (mMoveSharkElapsed < 0)
                {
                    mMoveSharkElapsed = (float)0.5;
                    int aDirection = mRandomGenerate.Next(1, 3);

                    if (aDirection == 1)
                    {
                        int sharkX = (int)mSprites.Sprites["SharkGuy1"].GridPositionX;
                        sharkX += mRandomGenerate.Next(-1, 2);
                        do
                        {
                            sharkX = (int)mSprites.Sprites["SharkGuy1"].GridPositionX;
                            int move = 0;
                            switch (mRandomGenerate.Next(1, 4))
                            {
                                case 1:
                                    move = -1;
                                    break;
                                case 2:
                                    move = 0;
                                    break;
                                case 3:
                                    move = 1;
                                    break;
                            }
                            sharkX += move;
                        } while (sharkX < 1 | sharkX > 3);

                        mSprites.Sprites["SharkGuy1"].GridPosition = new Vector2(sharkX, mSprites.Sprites["SharkGuy1"].GridPositionY);
                    }
                    else
                    {
                        int sharkY = (int)mSprites.Sprites["SharkGuy1"].GridPositionY;
                        sharkY += mRandomGenerate.Next(-1, 2);
                        do
                        {
                            sharkY = (int)mSprites.Sprites["SharkGuy1"].GridPositionY;
                            int move = 0;
                            switch (mRandomGenerate.Next(1, 4))
                            {
                                case 1:
                                    move = -1;
                                    break;
                                case 2:
                                    move = 0;
                                    break;
                                case 3:
                                    move = 1;
                                    break;
                            }
                            sharkY += move;
                        } while (sharkY < 1 | sharkY > 3);

                        mSprites.Sprites["SharkGuy1"].GridPosition = new Vector2(mSprites.Sprites["SharkGuy1"].GridPositionX, sharkY);
                    }
                }

                mSharkMouthElapsed -= elapsed;
                if (mSharkMouthElapsed < 0)
                {
                    mSharkMouthElapsed = (float)0.3;
                    mSprites.Sprites["SharkGuy1"].Visible = !mSprites.Sprites["SharkGuy1"].Visible;
                    mSprites.Sprites["SharkGuy2"].Visible = !mSprites.Sprites["SharkGuy2"].Visible;
                }
            }
        }

        //Update all of the text objects with their latest values
        protected void UpdateText()
        {
            //Update dynamic text values
            mDisplayText.Texts["LevelValue"].Value = mGameData.Level.ToString();
            mDisplayText.Texts["ScoreValue"].Value = mGameData.Score.ToString();
            mDisplayText.Texts["JellyBeansValue"].Value = mGameData.CustomData["JellyBeans"].ToString() + " OF " + mGameData.CustomData["JellyBeansMax"].ToString();
        }//End UpdateText()

        //Update the Sprite
        protected void UpdateSprite(float elapsed)
        {
            mMoveJellyElapsed -= elapsed;

            //Update the sprite
            if (mMoveJellyElapsed < 0)
            {
                mMoveJellyElapsed = (float)0.05;

                if (CurrentState != GameState.GameOver && mSprites.Sprites["Jelly"].Moving == true)
                {
                    switch (mCurrentJellyState)
                    {
                        //Sittting
                        case JellyState.Sitting:
                            {
                                mSprites.Sprites["Jelly"].TexturePositionX = 256;
                                mCurrentJellyState = JellyState.PreJump;
                                mJellyStateStartX = (int)mSprites.Sprites["Jelly"].Position.X;
                                mJellyStateStartY = (int)mSprites.Sprites["Jelly"].Position.Y;
                                break;
                            }

                        //Pre-Jump
                        case JellyState.PreJump:
                            {
                                mSprites.Sprites["Jelly"].TexturePositionX = 0;
                                mCurrentJellyState = JellyState.UpArc;
                                break;
                            }

                        //Up-Arc
                        case JellyState.UpArc:
                            {
                                mSprites.Sprites["Jelly"].TexturePositionX = 0;

                                if (mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Left)
                                {
                                    mSprites.Sprites["Jelly"].Position = new Vector2(mSprites.Sprites["Jelly"].Position.X - 30, mSprites.Sprites["Jelly"].Position.Y - 20);
                                }

                                else if (mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Right)
                                {
                                    mSprites.Sprites["Jelly"].Position = new Vector2(mSprites.Sprites["Jelly"].Position.X + 30, mSprites.Sprites["Jelly"].Position.Y - 20);
                                }
                                else if (mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Up)
                                {
                                    mSprites.Sprites["Jelly"].Position = new Vector2(mSprites.Sprites["Jelly"].Position.X + 20, mSprites.Sprites["Jelly"].Position.Y - 30);
                                }
                                else if (mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Down)
                                {
                                    mSprites.Sprites["Jelly"].Position = new Vector2(mSprites.Sprites["Jelly"].Position.X + 20, mSprites.Sprites["Jelly"].Position.Y + 30);
                                }

                                mCurrentJellyState = JellyState.DownArc;
                                break;
                            }

                        //Down-Arc
                        case JellyState.DownArc:
                            {
                                mSprites.Sprites["Jelly"].TexturePositionX = 256;
                                mCurrentJellyState = JellyState.Landing;

                                if (mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Left)
                                {
                                    mSprites.Sprites["Jelly"].Position = new Vector2(mSprites.Sprites["Jelly"].Position.X - 30, mSprites.Sprites["Jelly"].Position.Y + 10);


                                }

                                else if (mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Right)
                                {
                                    mSprites.Sprites["Jelly"].Position = new Vector2(mSprites.Sprites["Jelly"].Position.X + 30, mSprites.Sprites["Jelly"].Position.Y + 10);


                                }
                                else if (mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Up)
                                {
                                    mSprites.Sprites["Jelly"].Position = new Vector2(mSprites.Sprites["Jelly"].Position.X - 10, mSprites.Sprites["Jelly"].Position.Y - 30);


                                }
                                else if (mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Down)
                                {
                                    mSprites.Sprites["Jelly"].Position = new Vector2(mSprites.Sprites["Jelly"].Position.X - 10, mSprites.Sprites["Jelly"].Position.Y + 30);

                                }

                                break;
                            }

                        //Land
                        case JellyState.Landing:
                            {
                                mSprites.Sprites["Jelly"].TexturePositionX = 0;

                                if ((mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Left) | (mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Right))
                                {
                                    mSprites.Sprites["Jelly"].Position = new Vector2(mSprites.Sprites["Jelly"].Position.X, mJellyStateStartY);
                                }
                                else if ((mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Up) | (mSprites.Sprites["Jelly"].Direction == JelliesAPlenty.Sprite.Sprite.eDirection.Down))
                                {
                                    mSprites.Sprites["Jelly"].Position = new Vector2(mJellyStateStartX, mSprites.Sprites["Jelly"].Position.Y);
                                }

                                mSprites.Sprites["Jelly"].Direction = JelliesAPlenty.Sprite.Sprite.eDirection.None;
                                mSprites.Sprites["Jelly"].Moving = false;
                                mCurrentJellyState = JellyState.Sitting;

                                break;
                            }
                    }
                }





            }

        }

        //Check for keyboard input
        protected void UpdateKeyboard()
        {
            //Get the current state of the keyboard
            KeyboardState aKeyboard = Keyboard.GetState();

            //Get the current keys being pressed
            Keys[] aCurrentKeys = aKeyboard.GetPressedKeys();

            m_SpacePressed = false;

            //Cycle through all of the keys being pressed and move the sprite accordingly
            for (int aCurrentKey = 0; aCurrentKey < aCurrentKeys.Length; aCurrentKey++)
            {
                if (mSprites.Sprites["Jelly"].Moving == false && CurrentState != GameState.GameOver && mSprites.Sprites["NextLevel"].Elapsed == 0)
                {

                    //Change the sprite's direction to the left and move it left
                    if (aCurrentKeys[aCurrentKey] == Keys.Left)
                    {
                        if (mSprites.Sprites["Jelly"].GridPositionX > 1)
                        {
                            mSprites.Sprites["Jelly"].GridPositionX -= 1;
                            mSprites.Sprites["Jelly"].Moving = true;
                            mSprites.Sprites["Jelly"].Direction = JelliesAPlenty.Sprite.Sprite.eDirection.Left;

                        }
                    }

                    //Change the sprite's direction to the right and move it right
                    if (aCurrentKeys[aCurrentKey] == Keys.Right)
                    {
                        if (mSprites.Sprites["Jelly"].GridPositionX < 3)
                        {
                            mSprites.Sprites["Jelly"].GridPositionX += 1;
                            mSprites.Sprites["Jelly"].Moving = true;
                            mSprites.Sprites["Jelly"].Direction = JelliesAPlenty.Sprite.Sprite.eDirection.Right;


                        }
                    }

                    //Change the sprite's direction to up and move it up
                    if (aCurrentKeys[aCurrentKey] == Keys.Up)
                    {
                        if (mSprites.Sprites["Jelly"].GridPositionY > 1)
                        {
                            mSprites.Sprites["Jelly"].GridPositionY -= 1;
                            mSprites.Sprites["Jelly"].Moving = true;
                            mSprites.Sprites["Jelly"].Direction = JelliesAPlenty.Sprite.Sprite.eDirection.Up;


                        }

                    }

                    //Change the sprite's direction to down and move it down
                    if (aCurrentKeys[aCurrentKey] == Keys.Down)
                    {
                        if (mSprites.Sprites["Jelly"].GridPositionY < 3)
                        {
                            mSprites.Sprites["Jelly"].GridPositionY += 1;
                            mSprites.Sprites["Jelly"].Moving = true;
                            mSprites.Sprites["Jelly"].Direction = JelliesAPlenty.Sprite.Sprite.eDirection.Down;

                        }
                    }
                }

                //Exit the game if the Escape key has been pressed
                if (aCurrentKeys[aCurrentKey] == Keys.Escape)
                {
                    this.Exit();

                }

                if (CurrentState == GameState.GameOver)
                {
                    if (aCurrentKeys[aCurrentKey] == Keys.Space)
                    {
                        StartGame();
                    }
                }


                if (aCurrentKeys[aCurrentKey] == Keys.Enter)
                {
                    GenerateNextQuestion();
                }

                if (aCurrentKeys[aCurrentKey] == Keys.Space)
                {
                    m_SpacePressed = true;
                }
            }

            if (CurrentState != GameState.GameOver && mSprites.Sprites["NextLevel"].Elapsed == 0)
            {
                if (m_SpacePressed == true && m_SpacePressed != m_SpacePressedBefore)
                {
                    CheckAnswer();
                    //mAnswered = true;
                    m_SpacePressedBefore = m_SpacePressed;
                }
                else if (m_SpacePressed != m_SpacePressedBefore)
                {
                    m_SpacePressedBefore = m_SpacePressed;
                }
            }

        }

        protected void NextLevel()
        {
            mGameData.Level += 1;
            mGameData.CustomData["JellyBeans"] = 0;

            for (int index = 0; index < mGameData.CustomData["JellyBeansMax"]; index++)
            {
                string aKey = "JellyBean" + (index + 1).ToString();
                mSprites.Sprites[aKey].Visible = false;
            }

            mDisplayText.Texts["NextLevel"].Value = "LEVEL " + mGameData.Level.ToString();
            mDisplayText.Texts["NextLevel"].Visible = true;
            mSprites.Sprites["NextLevel"].Visible = true;
            
            mSprites.Sprites["Jelly"].GridPositionX = 1;
            mSprites.Sprites["Jelly"].GridPositionY = 1;
        }//End NextLevel()

        protected void GenerateNextQuestion()
        {
            mGameData.CustomData["QuestionPoints"] = 100;

            mDisplayText.Texts["QuestionValue1"].Value = mRandomGenerate.Next(0, 10).ToString();
            mDisplayText.Texts["QuestionValue2"].Value = mRandomGenerate.Next(0, 10).ToString();

            do
            {
                mDisplayText.Texts["QuestionValue1"].Value = mRandomGenerate.Next(0, 10).ToString();
                mDisplayText.Texts["QuestionValue2"].Value = mRandomGenerate.Next(0, 10).ToString();
            } while (int.Parse(mDisplayText.Texts["QuestionValue1"].Value) + int.Parse(mDisplayText.Texts["QuestionValue2"].Value) > 9);

            //Pick a random square to display the Correct answer in
            int aCorrectAnswerSquareIndex = mRandomGenerate.Next(0, 9);
            int aCorrectAnswer = int.Parse(mDisplayText.Texts["QuestionValue1"].Value) + int.Parse(mDisplayText.Texts["QuestionValue2"].Value);

            for (int aIndex = 0; aIndex < 9; aIndex++)
            {
                string aKey = "Square" + (aIndex + 1).ToString();
                if (aIndex == aCorrectAnswerSquareIndex)
                {
                    mDisplayText.Texts[aKey].Value = aCorrectAnswer.ToString();
                }
                else
                {
                    mDisplayText.Texts[aKey].Value = mRandomGenerate.Next(0, 10).ToString();
                    do
                    {
                        mDisplayText.Texts[aKey].Value = mRandomGenerate.Next(0, 10).ToString();
                    } while (mDisplayText.Texts[aKey].Value == aCorrectAnswer.ToString());
                }
            }
        }// End GenerateNextQuestion()

        protected void CheckAnswer()
        {
            int aCorrectAnswer = int.Parse(mDisplayText.Texts["QuestionValue1"].Value) + int.Parse(mDisplayText.Texts["QuestionValue2"].Value);
            if (mSprites.Sprites["Jelly"].GridPositionX == 1 && mSprites.Sprites["Jelly"].GridPositionY == 1)
            {
                if (mDisplayText.Texts["Square1"].Value == (aCorrectAnswer).ToString())
                {
                    CorrectAnswer();
                }
                else
                {
                    WrongAnswer();
                }
            }
            else if (mSprites.Sprites["Jelly"].GridPositionX == 1 && mSprites.Sprites["Jelly"].GridPositionY == 2)
            {
                if (mDisplayText.Texts["Square4"].Value == (aCorrectAnswer).ToString())
                {
                    CorrectAnswer();
                }
                else
                {
                    WrongAnswer();
                }
            }
            else if (mSprites.Sprites["Jelly"].GridPositionX == 1 && mSprites.Sprites["Jelly"].GridPositionY == 3)
            {
                if (mDisplayText.Texts["Square7"].Value == (aCorrectAnswer).ToString())
                {
                    CorrectAnswer();
                }
                else
                {
                    WrongAnswer();
                }
            }
            else if (mSprites.Sprites["Jelly"].GridPositionX == 2 && mSprites.Sprites["Jelly"].GridPositionY == 1)
            {
                if (mDisplayText.Texts["Square2"].Value == (aCorrectAnswer).ToString())
                {
                    CorrectAnswer();
                }
                else
                {
                    WrongAnswer();
                }
            }
            else if (mSprites.Sprites["Jelly"].GridPositionX == 2 && mSprites.Sprites["Jelly"].GridPositionY == 2)
            {
                if (mDisplayText.Texts["Square5"].Value == (aCorrectAnswer).ToString())
                {
                    CorrectAnswer();
                }
                else
                {
                    WrongAnswer();
                }
            }
            else if (mSprites.Sprites["Jelly"].GridPositionX == 2 && mSprites.Sprites["Jelly"].GridPositionY == 3)
            {
                if (mDisplayText.Texts["Square8"].Value == (aCorrectAnswer).ToString())
                {
                    CorrectAnswer();
                }
                else
                {
                    WrongAnswer();
                }
            }
            else if (mSprites.Sprites["Jelly"].GridPositionX == 3 && mSprites.Sprites["Jelly"].GridPositionY == 1)
            {
                if (mDisplayText.Texts["Square3"].Value == (aCorrectAnswer).ToString())
                {
                    CorrectAnswer();
                }
                else
                {
                    WrongAnswer();
                }
            }
            else if (mSprites.Sprites["Jelly"].GridPositionX == 3 && mSprites.Sprites["Jelly"].GridPositionY == 2)
            {
                if (mDisplayText.Texts["Square6"].Value == (aCorrectAnswer).ToString())
                {
                    CorrectAnswer();
                }
                else
                {
                    WrongAnswer();
                }
            }
            else if (mSprites.Sprites["Jelly"].GridPositionX == 3 && mSprites.Sprites["Jelly"].GridPositionY == 3)
            {
                if (mDisplayText.Texts["Square9"].Value == (aCorrectAnswer).ToString())
                {
                    CorrectAnswer();
                }
                else
                {
                    WrongAnswer();
                }
            }

        }

        protected void CorrectAnswer()
        {
            mGameData.Score += mGameData.CustomData["QuestionPoints"];
            mGameData.CustomData["JellyBeans"] += 1;
            DropJellyBean();
            CurrentState = GameState.Running;

            //mSharkReleased = false;

            if (mGameData.CustomData["JellyBeans"] == mGameData.CustomData["JellyBeansMax"])
            {
                NextLevel();
            }
            else
            {
                GenerateNextQuestion();
            }
        }

        protected void WrongAnswer()
        {
            if (CurrentState != GameState.SharkReleased)
            {
                mSprites.Sprites["SharkGuy1"].GridPositionX = 3;
                switch ((int)mSprites.Sprites["Jelly"].GridPositionY)
                {
                    case 1:
                        mSprites.Sprites["SharkGuy1"].GridPositionY = 3;
                        break;
                    case 2:
                        mSprites.Sprites["SharkGuy1"].GridPositionY = 3;
                        break;
                    case 3:
                        mSprites.Sprites["SharkGuy1"].GridPositionY = 1;
                        break;
                }
                CurrentState = GameState.SharkReleased;
                //mSharkReleased = true;
            }


            mGameData.CustomData["QuestionPoints"] -= 20;
            if (mGameData.CustomData["QuestionPoints"] < 0)
            {
                mGameData.CustomData["QuestionPoints"] = 0;
            }

            mSprites.Sprites["Warning"].Visible = true;

        }

        //Generate a new random color for the Color Square
        protected int GenerateColorSquare()
        {
            //Pick a color from the Color strip for the Colored Square in the Computer panel
            switch (mRandomGenerate.Next(0, 5))
            {
                case 0:
                    {
                        return 0;
                    }

                case 1:
                    {
                        return 31;
                    }

                case 2:
                    {
                        return 61;
                    }

                case 3:
                    {
                        return 93;
                    }

                case 4:
                    {
                        return 121;
                    }
            }

            return 0;
        }//End GenerateColorSquare()

        //Drop a JellyBean into the Jar
        protected void DropJellyBean()
        {
            //Get the Key for the Jellybean being dropped and set it's Sprite to Visible and Moving
            string aKey = "JellyBean" + mGameData.CustomData["JellyBeans"].ToString();
            mSprites.Sprites[aKey].Visible = true;
            mSprites.Sprites[aKey].Moving = true;

            //Pick one of the 5 JellyBean Colors and set the JellyBeans YPosition for the Texture appropriately
            switch (mRandomGenerate.Next(0, 5))
            {
                case 0:
                    {
                        mSprites.Sprites[aKey].TexturePositionY = 0;
                        break;
                    }
                case 1:
                    {
                        mSprites.Sprites[aKey].TexturePositionY = 95;
                        break;
                    }
                case 2:
                    {
                        mSprites.Sprites[aKey].TexturePositionY = 190;
                        break;
                    }
                case 3:
                    {
                        mSprites.Sprites[aKey].TexturePositionY = 285;
                        break;
                    }
                case 4:
                    {
                        mSprites.Sprites[aKey].TexturePositionY = 380;
                        break;
                    }
            }//End switch (mRandomGenerate.Next(0, 5))

            //Determine if the Jelly bean is falling in the left or the right side of the Jar
            if (mGameData.CustomData["JellyBeans"] - 1 < 5)
            {
                mSprites.Sprites[aKey].Position = new Vector2(20, 0);
            }
            else
            {
                mSprites.Sprites[aKey].Position = new Vector2(95, 0);
            }//End if (mJellyBeans - 1 < 5)

        }//End DropyJellyBean()


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            //Begin drawing the sprites. Using AlphaBlend as the blend mode will use the transparencies of the images drawn.
            spriteBatch.Begin();

            //Draw all of the sprite objects that are currently visible
            mSprites.Draw(spriteBatch);

            //Draw all of the text objects that are currently visible
            mDisplayText.Draw(spriteBatch);

            //Re-draw the sprites that need to be displayed over top of the text
            mSprites.Draw(spriteBatch, "Jelly");
            mSprites.Draw(spriteBatch, "SharkGuy1");
            mSprites.Draw(spriteBatch, "SharkGuy2");

            //Re-draw the Next Level text and display box if needed
            if (mSprites.Sprites["NextLevel"].Elapsed > 0)
            {
                mSprites.Draw(spriteBatch, "NextLevel");
                mDisplayText.Draw(spriteBatch, "NextLevel");
            }//End if (mSprites.Sprites["NextLevel"].Elapsed > 0)

            //Re-draw the Gamve Over text and dsplay box if needed
            if (CurrentState == GameState.GameOver)
            {
                mSprites.Sprites["GameOver"].Visible = true;
                mSprites.Draw(spriteBatch, "GameOver");
                mDisplayText.Draw(spriteBatch, "GameOver");
                mDisplayText.Draw(spriteBatch, "PressSpace");
            }//End if (CurrentState == GameState.GameOver)

            spriteBatch.End();

            base.Draw(gameTime);
        }//End Draw 
    }
}
